@@ -3,7 +3,6 @@ from pprint import pprint
 import sys
 import os
 from pathlib import Path
-from string import Template
 import re
 import shutil
 import textwrap
@@ -12,9 +11,13 @@ import tempfile
 from subprocess import CalledProcessError, run as sp_run
 from glob import glob
 
-from .config import Config
+from . import config
+from . import cmake
+from .datastructures import BuildPaths, PackageInfo
 
-from distlib.version import NormalizedVersion
+import flit_core.common  # type: ignore
+import flit_core.config  # type: ignore
+from distlib.version import NormalizedVersion  # type: ignore
 
 _CMAKE_MINIMUM_REQUIRED = NormalizedVersion('3.15')
 
@@ -24,7 +27,7 @@ class _BuildBackend(object):
     # --- Constructor ---------------------------------------------------------
 
     def __init__(self) -> None:
-        self.verbose = False
+        self.verbose: bool = False
 
     # --- Methods required by PEP 517 -----------------------------------------
 
@@ -32,7 +35,7 @@ class _BuildBackend(object):
         """https://www.python.org/dev/peps/pep-0517/#get-requires-for-build-wheel"""
         self.parse_config_settings(config_settings)
         pyproject = Path('pyproject.toml').resolve()
-        cfg = self.read_metadata(pyproject, config_settings)
+        cfg = self.read_config(pyproject, config_settings, self.verbose)
         deps = []
         # Check if we need CMake
         if cfg.cmake:
@@ -96,14 +99,14 @@ class _BuildBackend(object):
         # Load metadata
         from flit_core.common import Module, make_metadata
         pyproject = src_dir / 'pyproject.toml'
-        cfg = self.read_metadata(pyproject, config_settings)
+        cfg = self.read_config(pyproject, config_settings, self.verbose)
         import_name = cfg.module['name']
         pkg = Module(import_name, src_dir / cfg.module['directory'])
         metadata = make_metadata(pkg, cfg)
         metadata.version = self.normalize_version(metadata.version)
 
         # Export dist
-        from flit_core.sdist import SdistBuilder
+        from flit_core.sdist import SdistBuilder  # type: ignore
         rel_pyproject = os.path.relpath(pyproject, src_dir)
         extra_files = [str(rel_pyproject)] + cfg.referenced_files
         sdist_cfg = cfg.sdist[self.get_os_name()]
@@ -123,125 +126,158 @@ class _BuildBackend(object):
 
     # --- Parsing config options and metadata ---------------------------------
 
-    def parse_config_settings(self, config_settings: Optional[Dict]):
+    @staticmethod
+    def is_verbose_enabled(config_settings: Optional[dict]):
         if 'PY_BUILD_CMAKE_VERBOSE' in os.environ:
-            self.verbose = True
+            return True
         if config_settings is None:
-            return
-        if config_settings.keys() & {'verbose', '--verbose', 'v', '-v'}:
-            self.verbose = True
+            return False
+        if config_settings.keys() & {'verbose', '--verbose', 'V', '-V'}:
+            return True
+        return False
 
-    def read_metadata(self, pyproject, config_settings: Optional[Dict]):
-        from .config import read_metadata
-        from flit_core.config import ConfigError
+    def parse_config_settings(self, config_settings: Optional[Dict]):
+        self.verbose = self.is_verbose_enabled(config_settings)
 
+    @staticmethod
+    def read_config(pyproject_path: Path, config_settings: Optional[Dict],
+                    verbose: bool) -> config.Config:
         config_settings = config_settings or {}
         try:
+            if verbose:
+                print("Configuration settings:")
+                pprint(config_settings)
             listify = lambda x: x if isinstance(x, list) else [x]
             keys = ['--local', '--cross']
             overrides = {
                 key: listify(config_settings.get(key) or [])
                 for key in keys
             }
-            if self.verbose:
+            if verbose:
                 print("Configuration settings for local and "
                       "cross-compilation overrides:")
                 pprint(overrides)
-            cfg = read_metadata(pyproject, overrides)
-        except ConfigError as e:
+            cfg = config.read_config(pyproject_path, overrides)
+        except flit_core.config.ConfigError as e:
             e.args = ("\n"
                       "\n"
-                      "\t❌ Error in configuration:\n"
+                      "\t❌ Error in user configuration:\n"
                       "\n"
                       f"\t\t{e}\n"
                       "\n", )
             raise
-        if self.verbose:
-            self.print_config_verbose(cfg)
+        except AssertionError as e:
+            e.args = (
+                "\n"
+                "\n"
+                "\t❌ Internal error while processing the configuration\n"
+                "\t   Please notify the developers: https://github.com/tttapa/py-build-cmake/issues\n"
+                "\n"
+                f"\t\t{e}\n"
+                "\n", )
+            raise
+        if verbose:
+            _BuildBackend.print_config_verbose(cfg)
         return cfg
 
     # --- Building wheels -----------------------------------------------------
 
     def build_wheel_in_dir(self,
-                           wheel_directory,
+                           wheel_directory_,
                            tmp_build_dir,
                            config_settings,
                            editable=False):
         """This is the main function that contains all steps necessary to build
         a complete wheel package, including the CMake builds etc."""
         # Set up all paths
-        wheel_directory = Path(wheel_directory)
-        tmp_build_dir = Path(tmp_build_dir)
-        staging_dir = tmp_build_dir / 'staging'
-        src_dir = Path().resolve()
+        paths = BuildPaths(
+            source_dir=Path().resolve(),
+            wheel_dir=Path(wheel_directory_),
+            temp_dir=Path(tmp_build_dir),
+            staging_dir=Path(tmp_build_dir) / 'staging',
+        )
 
         # Load metadata from the pyproject.toml file
-        from .config import normalize_name_wheel
-        from flit_core.common import Module, make_metadata
-        cfg = self.read_metadata(src_dir / 'pyproject.toml', config_settings)
-        dist_name = normalize_name_wheel(cfg.metadata['name'])
-        import_name = cfg.module['name']
-        pkg = Module(import_name, src_dir / cfg.module['directory'])
-        metadata = make_metadata(pkg, cfg)
-        metadata.version = self.normalize_version(metadata.version)
+        cfg, pkg, metadata = _BuildBackend.read_all_metadata(
+            paths.source_dir, config_settings, self.verbose)
+
+        pkg_info = PackageInfo(
+            version=metadata.version,
+            package_name=cfg.package_name,
+            module_name=cfg.module['name'],
+        )
 
         # Copy the module's Python source files to the temporary folder
         if not editable:
-            self.copy_pkg_source_to(staging_dir, src_dir, pkg)
+            self.copy_pkg_source_to(paths.staging_dir, pkg)
         else:
-            self.write_editable_wrapper(staging_dir, src_dir, pkg)
+            self.write_editable_wrapper(paths.staging_dir, pkg)
 
         # Create dist-info folder
-        distinfo = staging_dir / f'{dist_name}-{metadata.version}.dist-info'
-        os.makedirs(distinfo, exist_ok=True)
+        distinfo_dir = f'{pkg_info.package_name}-{pkg_info.version}.dist-info'
+        distinfo_dir = paths.staging_dir / distinfo_dir
+        os.makedirs(distinfo_dir, exist_ok=True)
 
         # Write metadata
-        metadata_path = distinfo / 'METADATA'
+        metadata_path = distinfo_dir / 'METADATA'
         with open(metadata_path, 'w', encoding='utf-8') as f:
             metadata.write_metadata_file(f)
         # Write or copy license
-        self.write_license_files(cfg.license, src_dir, distinfo)
+        self.write_license_files(cfg.license, paths.source_dir, distinfo_dir)
         # Write entrypoints/scripts
-        self.write_entrypoints(distinfo, cfg)
+        self.write_entrypoints(distinfo_dir, cfg)
 
         # Generate .pyi stubs (for the Python files only)
         if cfg.stubgen is not None and not editable:
-            self.generate_stubs(tmp_build_dir, staging_dir, pkg, cfg.stubgen)
+            self.generate_stubs(paths, pkg, cfg.stubgen)
 
         # Configure, build and install the CMake project
         if cfg.cmake:
-            self.do_native_cross_cmake_build(tmp_build_dir, staging_dir,
-                                             src_dir, cfg, metadata, dist_name,
-                                             import_name)
+            self.do_native_cross_cmake_build(paths, cfg, pkg_info)
 
         # Create wheel
-        whl_name = self.create_wheel(wheel_directory, staging_dir, cfg,
-                                     dist_name, metadata.version)
+        whl_name = self.create_wheel(paths, cfg, pkg_info)
         return whl_name
 
-    def create_wheel(self, wheel_directory, tmp_build_dir, cfg, dist_name,
-                     norm_version):
+    @staticmethod
+    def read_all_metadata(src_dir, config_settings, verbose):
+        from flit_core.common import Module, make_metadata
+        cfg = _BuildBackend.read_config(src_dir / 'pyproject.toml',
+                                        config_settings, verbose)
+        pkg = Module(cfg.module['name'], src_dir / cfg.module['directory'])
+        metadata = make_metadata(pkg, cfg)
+        metadata.version = _BuildBackend.normalize_version(metadata.version)
+        return cfg, pkg, metadata
+
+    def create_wheel(self, paths, cfg, package_info):
         """Create a wheel package from the build directory."""
-        from distlib.wheel import Wheel
+        from distlib.wheel import Wheel  # type: ignore
         whl = Wheel()
-        whl.name = dist_name
-        whl.version = norm_version
+        whl.name = package_info.package_name
+        whl.version = package_info.version
         pure = not cfg.cmake
         libdir = 'purelib' if pure else 'platlib'
-        paths = {'prefix': str(tmp_build_dir), libdir: str(tmp_build_dir)}
-        whl.dirname = wheel_directory
+        staging_dir = paths.staging_dir
+        whl_paths = {'prefix': str(staging_dir), libdir: str(staging_dir)}
+        whl.dirname = paths.wheel_dir
         if pure:
             tags = {'pyver': ['py3']}
         elif cfg.cross:
             tags = self.get_cross_tags(cfg.cross)
         else:
             tags = self.get_native_tags()
-        wheel_path = whl.build(paths, tags=tags, wheel_version=(1, 0))
-        whl_name = os.path.relpath(wheel_path, wheel_directory)
+        wheel_path = whl.build(whl_paths, tags=tags, wheel_version=(1, 0))
+        whl_name = os.path.relpath(wheel_path, paths.wheel_dir)
         return whl_name
 
-    def do_native_cross_cmake_build(self, tmp_build_dir, staging_dir, src_dir,
-                                    cfg, metadata, dist_name, import_name):
+    @staticmethod
+    def get_cmake_configs(cfg):
+        native_cmake_cfg = cfg.cmake[_BuildBackend.get_os_name()]
+        cmake_cfg = cfg.cmake['cross'] if cfg.cross else native_cmake_cfg
+        return cmake_cfg, native_cmake_cfg
+
+    def do_native_cross_cmake_build(self, paths: BuildPaths, cfg,
+                                    package_info):
         """If not cross-compiling, just do a regular CMake build+install.
         When cross-compiling, do a cross-build+install (using the provided 
         CMake toolchain file).
@@ -251,21 +287,20 @@ class _BuildBackend(object):
         from the native build that match the patterns in
         cfg.cross['copy_from_native_build'] are copied to the installation
         directory of the cross-build for packaging."""
+        cmake_cfg, native_cmake_cfg = self.get_cmake_configs(cfg)
         # When cross-compiling, optionally do a native build first
         native_install_dir = None
-        native_cmake_cfg = cfg.cmake[self.get_os_name()]
         if self.needs_cross_native_build(cfg):
-            native_install_dir = tmp_build_dir / 'native-install'
-            self.run_cmake(src_dir, native_install_dir, metadata,
-                           native_cmake_cfg, None, native_install_dir,
-                           dist_name, import_name)
+            native_install_dir = paths.temp_dir / 'native-install'
+            self.run_cmake(paths.source_dir, native_install_dir,
+                           native_cmake_cfg, None, package_info,
+                           native_install_dir)
         # Then do the actual build
-        cmake_cfg = cfg.cmake['cross'] if cfg.cross else native_cmake_cfg
-        self.run_cmake(src_dir, staging_dir, metadata, cmake_cfg, cfg.cross,
-                       native_install_dir, dist_name, import_name)
+        self.run_cmake(paths.source_dir, paths.staging_dir, cmake_cfg,
+                       cfg.cross, package_info, native_install_dir)
         # Finally, move the files from the native build to the staging area
         if native_install_dir:
-            self.copy_native_install(staging_dir, native_install_dir,
+            self.copy_native_install(paths.staging_dir, native_install_dir,
                                      cfg.cross['copy_from_native_build'])
 
     def copy_native_install(self, staging_dir, native_install_dir,
@@ -289,11 +324,12 @@ class _BuildBackend(object):
 
     # --- Files installation --------------------------------------------------
 
-    def copy_pkg_source_to(self, tmp_build_dir, src_dir, pkg):
+    def copy_pkg_source_to(self, staging_dir: Path,
+                           pkg: flit_core.common.Module):
         """Copy the files of a Python package to the build directory."""
         for mod_file in pkg.iter_files():
             rel_path = os.path.relpath(mod_file, pkg.path.parent)
-            dst = tmp_build_dir / rel_path
+            dst = staging_dir / rel_path
             os.makedirs(dst.parent, exist_ok=True)
             shutil.copy2(mod_file, dst, follow_symlinks=False)
 
@@ -306,17 +342,18 @@ class _BuildBackend(object):
         elif 'file' in license:
             shutil.copy2(srcdir / license['file'], distinfo_dir)
 
-    def write_entrypoints(self, distinfo: Path, cfg: Config):
+    def write_entrypoints(self, distinfo: Path, cfg: config.Config):
         from flit_core.common import write_entry_points
         with (distinfo / 'entry_points.txt').open('w', encoding='utf-8') as f:
             write_entry_points(cfg.entrypoints, f)
 
     # --- Editable installs ---------------------------------------------------
 
-    def write_editable_wrapper(self, tmp_build_dir: Path, src_dir: Path, pkg):
+    def write_editable_wrapper(self, staging_dir: Path,
+                               pkg: flit_core.common.Module):
         """Write a fake __init__.py file that points to the development 
         folder."""
-        tmp_pkg: Path = tmp_build_dir / pkg.name
+        tmp_pkg: Path = staging_dir / pkg.name
         pkgpath = Path(pkg.path)
         initpath = pkgpath / '__init__.py'
         os.makedirs(tmp_pkg, exist_ok=True)
@@ -348,128 +385,85 @@ class _BuildBackend(object):
         if py_typed.exists():
             shutil.copy2(py_typed, tmp_pkg)
         # Write a path file so IDEs find the correct files as well
-        (tmp_build_dir / f'{pkg.name}.pth').write_text(str(pkg.path.parent))
+        (staging_dir / f'{pkg.name}.pth').write_text(str(pkg.path.parent))
 
     # --- Invoking CMake builds -----------------------------------------------
 
-    def run_cmake(self, pkgdir, install_dir, metadata, cmake_cfg, cross_cfg,
-                  native_install_dir, dist_name, import_name):
+    def run_cmake(self, pkgdir, install_dir, cmake_cfg, cross_cfg,
+                  package_info, native_install_dir):
         """Configure, build and install using CMake."""
-        # Source and build folders
-        srcdir = Path(cmake_cfg.get('source_path', pkgdir)).resolve()
-        builddir = pkgdir / '.py-build-cmake_cache'
-        builddir = Path(cmake_cfg.get('build_path', builddir))
-        if not builddir.is_absolute():
-            builddir = pkgdir / builddir
-        buildconfig = self.get_build_config_name(cross_cfg)
-        builddir = builddir / buildconfig
-        builddir = builddir.resolve()
-        # Environment variables
-        cmake_env = os.environ.copy()
-        f = 'env'
-        if f in cmake_cfg:
-            for k, v in cmake_cfg[f].items():
-                cmake_env[k] = Template(v).substitute(cmake_env)
 
-        # Configure
-        configure_cmd = [
-            'cmake',
-            '-B',
-            str(builddir),
-            '-S',
-            str(srcdir),
-            '-D',
-            'PY_BUILD_CMAKE_PACKAGE_VERSION:STRING=' + metadata.version,
-            '-D',
-            'PY_BUILD_CMAKE_PACKAGE_NAME:STRING=' + dist_name,
-            '-D',
-            'PY_BUILD_CMAKE_MODULE_NAME:STRING=' + import_name,
-        ]
+        cmaker = self.get_cmaker(pkgdir,
+                                 install_dir,
+                                 cmake_cfg,
+                                 cross_cfg,
+                                 native_install_dir,
+                                 package_info,
+                                 verbose=self.verbose)
+
+        cmaker.configure()
+        cmaker.build()
+        cmaker.install()
+
+    @staticmethod
+    def get_cmaker(pkg_dir: Path, install_dir: Optional[Path], cmake_cfg: dict,
+                   cross_cfg: Optional[dict],
+                   native_install_dir: Optional[Path],
+                   package_info: PackageInfo, **kwargs):
+        toolchain_file = None
         if cross_cfg:
-            toolchain = (pkgdir / cross_cfg['toolchain_file']).resolve()
-            configure_cmd += [
-                '-D', 'CMAKE_TOOLCHAIN_FILE:FILEPATH=' + str(toolchain), '-D',
-                'Python3_EXECUTABLE:FILEPATH=' + sys.executable
-            ]
-        else:
-            configure_cmd += [
-                '-D', 'Python3_EXECUTABLE:FILEPATH=' + sys.executable, '-D',
-                'Python3_ROOT_DIR:PATH=' + sys.prefix, '-D',
-                'Python3_FIND_REGISTRY=NEVER', '-D',
-                'Python3_FIND_STRATEGY=LOCATION'
-            ]
-            # Cross-compile support for macOS - respect ARCHFLAGS if set
-            # https://github.com/pybind/cmake_example/pull/48
-            if self.get_os_name() == "mac" and "ARCHFLAGS" in os.environ:
-                archs = re.findall(r"-arch (\S+)", os.environ["ARCHFLAGS"])
-                if archs:
-                    configure_cmd += [
-                        '-D',
-                        'CMAKE_OSX_ARCHITECTURES={}'.format(";".join(archs)),
-                    ]
+            toolchain_file = (pkg_dir / cross_cfg['toolchain_file']).resolve()
+
+        # Add some CMake configure options
+        options = cmake_cfg.get('options', {})
         if native_install_dir:
-            configure_cmd += [
-                '-D', 'PY_BUILD_CMAKE_NATIVE_INSTALL_DIR:PATH=' +
+            options['PY_BUILD_CMAKE_NATIVE_INSTALL_DIR:PATH'] = \
                 str(native_install_dir)
-            ]
-        configure_cmd += cmake_cfg.get('args', [])  # User-supplied arguments
-        for k, v in cmake_cfg.get('options', {}).items():  # -D {option}={val}
-            configure_cmd += ['-D', k + '=' + v]
         btype = cmake_cfg.get('build_type')
         if btype:  # -D CMAKE_BUILD_TYPE={type}
-            configure_cmd += ['-D', 'CMAKE_BUILD_TYPE=' + btype]
-        gen = cmake_cfg.get('generator')
-        if gen:  # -G {generator}
-            configure_cmd += ['-G', gen]
-        self.run(configure_cmd, check=True, env=cmake_env)
+            options['CMAKE_BUILD_TYPE:STRING'] = btype
 
-        # Build and install
-        if not cmake_cfg.get('config', False):
-            self.build_install_cmake(cmake_cfg, builddir, cmake_env,
-                                     install_dir, None)
-        else:
-            for config in cmake_cfg['config']:
-                self.build_install_cmake(cmake_cfg, builddir, cmake_env,
-                                         install_dir, config)
+        # Build folder for each platform
+        build_cfg_name = _BuildBackend.get_build_config_name(cross_cfg)
 
-    def build_install_cmake(self, cmake_cfg, builddir, cmake_env, install_dir,
-                            config):
-        """Build the CMake project and install it."""
-        self.build_cmake(cmake_cfg, builddir, cmake_env, config)
-        self.install_cmake(cmake_cfg, builddir, cmake_env, install_dir, config)
-
-    def build_cmake(self, cmake_cfg, builddir, cmake_env, config):
-        """Build the CMake project."""
-        build_cmd = ['cmake', '--build', str(builddir)]
-        build_cmd += cmake_cfg.get('build_args', [])  # User-supplied arguments
-        if config:  # --config {config}
-            build_cmd += ['--config', config]
-        f = 'build_tool_args'
-        if f in cmake_cfg:
-            build_cmd += ['--'] + cmake_cfg[f]
-        self.run(build_cmd, check=True, env=cmake_env)
-
-    def install_cmake(self, cmake_cfg, builddir, cmake_env, install_dir,
-                      config):
-        """Install the CMake project."""
-        for component in cmake_cfg['install_components']:
-            install_cmd = [
-                'cmake', '--install',
-                str(builddir), '--prefix',
-                str(install_dir)
-            ]
-            install_cmd += cmake_cfg.get('install_args', [])
-            if config:  # --config {config}
-                install_cmd += ['--config', config]
-            if component:
-                install_cmd += ['--component', component]
-            print('Installing component', component or 'all')
-            self.run(install_cmd, check=True, env=cmake_env)
+        # CMake options
+        return cmake.CMaker(
+            cmake_settings=cmake.CMakeSettings(
+                working_dir=Path(pkg_dir),
+                source_path=Path(cmake_cfg["source_path"]),
+                build_path=Path(cmake_cfg['build_path']) / build_cfg_name,
+                os=_BuildBackend.get_os_name(),
+                command=Path("cmake"),
+            ),
+            conf_settings=cmake.CMakeConfigureSettings(
+                environment=cmake_cfg.get("env", {}),
+                toolchain_file=toolchain_file,
+                build_type=cmake_cfg.get('build_type'),
+                options=options,
+                args=cmake_cfg.get('args', []),
+                preset=cmake_cfg.get('preset'),
+                generator=cmake_cfg.get('generator'),
+            ),
+            build_settings=cmake.CMakeBuildSettings(
+                args=cmake_cfg['build_args'],
+                tool_args=cmake_cfg['build_tool_args'],
+                presets=cmake_cfg.get('build_presets', []),
+                configs=cmake_cfg.get('config', []),
+            ),
+            install_settings=cmake.CMakeInstallSettings(
+                args=cmake_cfg['install_args'],
+                presets=cmake_cfg.get('install_presets', []),
+                configs=cmake_cfg.get('config', []),
+                components=cmake_cfg.get('install_components', []),
+                prefix=install_dir,
+            ),
+            package_info=package_info,
+            **kwargs,
+        )
 
     # --- Generate stubs ------------------------------------------------------
 
-    def generate_stubs(self, tmp_build_dir, staging_dir, pkg, cfg: Dict[str,
-                                                                        Any]):
+    def generate_stubs(self, paths: BuildPaths, pkg, cfg: Dict[str, Any]):
         """Generate stubs (.pyi) using mypy stubgen."""
         args = ['stubgen'] + cfg.get('args', [])
         cfg.setdefault('packages', [pkg.name] if pkg.is_package else [])
@@ -481,9 +475,9 @@ class _BuildBackend(object):
         args += cfg.get('files', [])
         # Add output folder argument if not already specified in cfg['args']
         if 'args' not in cfg or not ({'-o', '--output'} & set(cfg['args'])):
-            args += ['-o', str(staging_dir)]
+            args += ['-o', str(paths.staging_dir)]
         env = os.environ.copy()
-        env.setdefault('MYPY_CACHE_DIR', str(tmp_build_dir))
+        env.setdefault('MYPY_CACHE_DIR', str(paths.temp_dir))
         # Call mypy stubgen in a subprocess
         self.run(args, cwd=pkg.path.parent, check=True, env=env)
 
@@ -497,7 +491,8 @@ class _BuildBackend(object):
             "Darwin": "mac",  # TODO: untested
         }[platform.system()]
 
-    def print_config_verbose(self, cfg):
+    @staticmethod
+    def print_config_verbose(cfg):
         print("\npy-build-cmake options")
         print("======================")
         print("module:")
@@ -512,7 +507,8 @@ class _BuildBackend(object):
         pprint(cfg.cross)
         print("======================\n")
 
-    def normalize_version(self, version):
+    @staticmethod
+    def normalize_version(version):
         from distlib.version import NormalizedVersion
         norm_version = str(NormalizedVersion(version))
         return norm_version
@@ -524,17 +520,9 @@ class _BuildBackend(object):
             pprint(kwargs)
         return sp_run(*args, **kwargs)
 
-    @staticmethod
-    def get_cross_tags(crosscfg):
-        """Get the PEP 425 tags to use when cross-compiling."""
-        return {
-            'pyver': [crosscfg['implementation'] + crosscfg['version']],
-            'abi': [crosscfg['abi']],
-            'arch': [crosscfg['arch']],
-        }
-
     @staticmethod
     def get_native_tags():
+        """Get the PEP 425 tags for the current platform."""
         from .tags import get_python_tag, get_abi_tag, get_platform_tag
         return {
             'pyver': [get_python_tag()],
@@ -542,6 +530,18 @@ class _BuildBackend(object):
             'arch': [get_platform_tag()],
         }
 
+    @staticmethod
+    def get_cross_tags(crosscfg):
+        """Get the PEP 425 tags to use when cross-compiling."""
+        tags = _BuildBackend.get_native_tags()
+        if 'implementation' in crosscfg and 'version' in crosscfg:
+            tags['pyver'] = [crosscfg['implementation'] + crosscfg['version']]
+        if 'abi' in crosscfg:
+            tags['abi'] = [crosscfg['abi']]
+        if 'arch' in crosscfg:
+            tags['arch'] = [crosscfg['arch']]
+        return tags
+
     @staticmethod
     def get_build_config_name(cross_cfg):
         """Get a string representing the Python version, ABI and architecture,
@@ -553,10 +553,12 @@ class _BuildBackend(object):
             tags = _BuildBackend.get_native_tags()
         return '-'.join(map(lambda x: x[0], tags.values()))
 
-    def needs_cross_native_build(self, cfg):
+    @staticmethod
+    def needs_cross_native_build(cfg):
         return cfg.cross and 'copy_from_native_build' in cfg.cross
 
-    def iter_files(self, stagedir):
+    @staticmethod
+    def iter_files(stagedir):
         """Iterate over the files contained in the given folder.
 
         Yields absolute paths - caller may want to make them relative.
@@ -579,7 +581,7 @@ class _BuildBackend(object):
 
     # --- Helper functions for finding programs like CMake --------------------
 
-    def check_cmake_program(self, cfg: Config, deps: List[str]):
+    def check_cmake_program(self, cfg: config.Config, deps: List[str]):
         assert cfg.cmake
         # Do we need to perform a native build?
         native = not cfg.cross or self.needs_cross_native_build(cfg)
@@ -611,7 +613,7 @@ class _BuildBackend(object):
             if not self.check_program_version('ninja', None, "Ninja"):
                 deps.append("ninja")
 
-    def check_stubgen_program(self, cfg: Config, deps: List[str]):
+    def check_stubgen_program(self, cfg: config.Config, deps: List[str]):
         if not self.check_program_version('stubgen', None, None, False):
             deps.append("mypy")
 
